library account;

import 'dart:core';
import 'package:json_annotation/json_annotation.dart';
import 'provider/authing.dart';

part 'client.g.dart';

enum Gender {
  @JsonValue("M")
  male,
  @JsonValue("W")
  female,
  @JsonValue("U")
  unknown;

  String getValue() {
    return _$GenderEnumMap[this] ?? "U";
  }
}

@JsonSerializable(fieldRename: FieldRename.snake, includeIfNull: false)
class Address {
  final String? country;
  final String? province;
  final String? city;
  final String? street;

  Address([this.country, this.province, this.city, this.street]);

  factory Address.fromJson(Map<String, dynamic> json) =>
      _$AddressFromJson(json);

  Map<String, dynamic> toJson() => _$AddressToJson(this);
}

@JsonSerializable(fieldRename: FieldRename.snake, includeIfNull: false)
class Account {
  final String userId;
  final String domain;
  final String tenantId;
  final DateTime createdAt;
  final String? username;
  final String? phone;
  final String? email;

  final String? nickname;
  final String? givenName;
  final String? familyName;
  final String? middleName;
  final String? photo;
  final Gender? gender;
  final DateTime? birthdate;
  final Address? address;
  final String? brief;

  Account(this.userId, this.domain, this.tenantId, this.createdAt,
      [this.username,
      this.phone,
      this.email,
      this.nickname,
      this.givenName,
      this.familyName,
      this.middleName,
      this.photo,
      this.gender,
      this.birthdate,
      this.address,
      this.brief]);

  factory Account.fromJson(Map<String, dynamic> json) =>
      _$AccountFromJson(json);

  Map<String, dynamic> toJson() => _$AccountToJson(this);
}

class AddressBuilder {
  String? _country;
  String? _province;
  String? _city;
  String? _street;

  AddressBuilder.clone(Address from)
      : _country = from.country,
        _province = from.province,
        _city = from.city,
        _street = from.street;

  AddressBuilder withCountry(String? country) {
    _country = country;
    return this;
  }

  AddressBuilder withProvince(String? province) {
    _province = province;
    return this;
  }

  AddressBuilder withCity(String? city) {
    _city = city;
    return this;
  }

  AddressBuilder withStreet(String? street) {
    _street = street;
    return this;
  }

  Address build() => Address(_country, _province, _city, _street);
}

class AccountBuilder {
  final String _userId;
  final String _domain;
  final String _tenantId;
  final DateTime _createdAt;
  final String? _username;
  final String? _phone;
  final String? _email;

  String? _nickname;
  String? _givenName;
  String? _familyName;
  String? _middleName;
  String? _photo;
  Gender? _gender;
  DateTime? _birthdate;
  Address? _address;
  String? _brief;

  AccountBuilder.clone(Account from)
      : _userId = from.userId,
        _domain = from.domain,
        _tenantId = from.tenantId,
        _createdAt = from.createdAt,
        _username = from.username,
        _phone = from.phone,
        _email = from.email,
        _nickname = from.nickname,
        _givenName = from.givenName,
        _familyName = from.familyName,
        _middleName = from.middleName,
        _photo = from.photo,
        _gender = from.gender,
        _birthdate = from.birthdate,
        _address = from.address,
        _brief = from.brief;

  AccountBuilder withNickname(String nickname) {
    _nickname = nickname;
    return this;
  }

  AccountBuilder withGivenName(String givenName) {
    _givenName = givenName;
    return this;
  }

  AccountBuilder withFamilyName(String familyName) {
    _familyName = familyName;
    return this;
  }

  AccountBuilder withMiddleName(String middleName) {
    _middleName = middleName;
    return this;
  }

  AccountBuilder withPhoto(String photo) {
    _photo = photo;
    return this;
  }

  AccountBuilder withGender(Gender gender) {
    _gender = gender;
    return this;
  }

  AccountBuilder withBirthdate(DateTime birthdate) {
    _birthdate = birthdate;
    return this;
  }

  AccountBuilder withAddress(Address address) {
    _address = address;
    return this;
  }

  AccountBuilder withBrief(String brief) {
    _brief = brief;
    return this;
  }

  Account build() => Account(
        _userId,
        _domain,
        _tenantId,
        _createdAt,
        _username,
        _phone,
        _email,
        _nickname,
        _givenName,
        _familyName,
        _middleName,
        _photo,
        _gender,
        _birthdate,
        _address,
        _brief,
      );
}

@JsonSerializable(fieldRename: FieldRename.snake, includeIfNull: false)
class AuthInfo {
  final Account account;
  final String idToken;
  final String? accessToken;
  final String? refreshToken;

  AuthInfo(this.account, this.idToken, [this.accessToken, this.refreshToken]);

  factory AuthInfo.fromJson(Map<String, dynamic> json) =>
      _$AuthInfoFromJson(json);

  Map<String, dynamic> toJson() => _$AuthInfoToJson(this);
}

enum StatusCode implements Comparable<StatusCode> {
  @JsonValue(0)
  none(value: 0),
  @JsonValue(200)
  ok(value: 200),
  @JsonValue(400)
  badRequest(value: 400),
  @JsonValue(401)
  unauthorized(value: 401),
  @JsonValue(402)
  paymentRequired(value: 402),
  @JsonValue(403)
  forbidden(value: 403),
  @JsonValue(404)
  notFound(value: 404),
  @JsonValue(405)
  methodNotAllowed(value: 405),
  @JsonValue(407)
  oauthRequired(value: 407),
  @JsonValue(409)
  conflict(value: 409),
  @JsonValue(410)
  gone(value: 410),
  @JsonValue(429)
  tooManyRequests(value: 429),
  @JsonValue(500)
  internalServerError(value: 500),
  @JsonValue(503)
  serviceUnavailable(value: 503),
  @JsonValue(505)
  versionNotSupported(value: 505),
  @JsonValue(600)
  localSdkError(value: 600);

  const StatusCode({
    required this.value,
  });

  final int value;

  @override
  int compareTo(StatusCode other) => value - other.value;
}

@JsonSerializable(
    genericArgumentFactories: true,
    fieldRename: FieldRename.snake,
    createFactory: false,
    includeIfNull: false)
class Result<TData> {
  final StatusCode code;
  final String message;
  final TData? data;

  Result(this.code, this.message, [this.data]);

  Result.success(this.data)
      : code = StatusCode.ok,
        message = "";

  Result.failure(this.code, this.message) : data = null;

  Map<String, dynamic> toJson() => _$ResultToJson(this, (TData v) => v);

  bool isSuccess() {
    return code == StatusCode.ok;
  }
}

abstract class OddsAccountClient {
  factory OddsAccountClient.authing(String userPoolId, String appId,
      [String? redirectBackUri = "https://oddshorizon.com/home",
      String? appHost = "https://oddshorizon.authing.cn"]) {
    final authing = Authing.singleton();
    authing.userPoolId = userPoolId;
    authing.appId = appId;
    authing.redirectBackUri = redirectBackUri;
    authing.appHost = appHost;

    return authing;
  }

  Future init();

  Future<Result<void>> claimPhoneOtp(String phone, [String? countryCode]);

  Future<Result<AuthInfo>> registerByPhoneWithOtp(
      String phone, String password, String otp);

  Future<Result<AuthInfo>> registerByUsername(String username, String password);

  Future<Result<AuthInfo>> registerByEmail(String email, String password);

  Future<Result<AuthInfo>> loginByPhoneOtp(String phone, String otp);

  Future<Result<AuthInfo>> loginByPassword(String account, String password);

  Future<Result<void>> logout();

  OidcAuthSession createOidcAuthSession();

  Future<Result<AuthInfo>> renewToken(String refreshToken);

  Future<Result<String>> uploadAvatar(String photoFilePath);

  Future<Result<Account>> updateProfile(Account profile);

  Future<Result<void>> updateBrief(String brief);

  Future<Result<AuthInfo>> resetPassword(String newPassword,
      [String? oldPassword]);

  Future<Result<AuthInfo>> resetPasswordByPhoneOtp(
      String phone, String otp, String newPassword);
}

abstract class OidcAuthSession {
  Future<String> getAuthPageUri();

  Future<Result<AuthInfo>> obtainAuthInfoByRedirectUri(String redirectBackUri);
}
